#include "cAnimation.h"

#include "cImage.h"

#define NOT_USED    -1

cAnimation::cAnimation()
{
    xOffset = yOffset = 0;
    m_length = 0;
    m_delay = 0;
    m_numPics = 0;
    m_loaded = false;
    m_repeat = false;

    for (int i = 0; i < MAX_CONCURRENT; i++) {
        m_currentFrame[i] = NOT_USED;
    }

    for (int i = 0; i < MAX_PICS; i++) {
        m_pic[i] = NULL;
    }
}

cAnimation::cAnimation(char *filename)
{
    xOffset = yOffset = 0;
    m_length = 0;
    m_delay = 0;
    m_numPics = 0;
    m_loaded = false;
    m_repeat = false;

    for (int i = 0; i < MAX_CONCURRENT; i++) {
        m_currentFrame[i] = NOT_USED;
    }

    for (int i = 0; i < MAX_PICS; i++) {
        m_pic[i] = NULL;
    }

    Load(filename);
}

cAnimation::~cAnimation()
{
	Unload();
}

int
cAnimation::Play(int x, int y)
{
    int animIndex;

    for (animIndex = 0; animIndex < MAX_CONCURRENT; animIndex++) {
        if (m_currentFrame[animIndex] == NOT_USED) {
            break;
        }
    }

    // failure
    if (animIndex >= MAX_CONCURRENT) {
        printf ("MAX_CONCURRENT too small\n");
        return -1;
    }

    m_currentFrame[animIndex] = 0;
    m_x[animIndex] = x;
    m_y[animIndex] = y;

    return (animIndex);
}

int
cAnimation::Play(int x, int y, int startFrame)
{
	int animIndex;

	animIndex = Play(x, y);
	m_currentFrame[animIndex] = startFrame % m_length;

	return animIndex;
}

void
cAnimation::Stop()
{
    for (int i = 0; i < MAX_CONCURRENT; i++)
        m_currentFrame[i] = NOT_USED;
}

void
cAnimation::Stop(int animIndex)
{
    m_currentFrame[animIndex] = NOT_USED;
}

void
cAnimation::setOffset(int x, int y)
{
    xOffset = x;
    yOffset = y;
}

void
cAnimation::set(int *frames, int delay, int length, bool repeat)
{
    for (int i = 0; i < length; i++) {
        m_frame[i] = frames[i];
    }
    m_length = length;
    m_repeat = repeat;

    m_delay = delay;
    if (m_delay < 1)
        m_delay = 1;
}

void
cAnimation::Update()
{
    if ((framecounter % m_delay) == 0) {
        for (int i = 0; i < MAX_CONCURRENT; i++) {
            if (m_currentFrame[i] != NOT_USED) {
                ++m_currentFrame[i];
                if (m_currentFrame[i] >= m_length) {
                    if (m_repeat)
                        m_currentFrame[i] = 0;
                    else
                        m_currentFrame[i] = NOT_USED;
                }
            }
        }
    }
}

void
cAnimation::Draw()
{
    for (int i = 0; i < MAX_CONCURRENT; i++) {
        if (m_currentFrame[i] != NOT_USED) {
            m_pic[m_frame[m_currentFrame[i]]]->Draw(m_x[i]-xOffset, m_y[i]-yOffset);
        }
    }
}

bool
cAnimation::addImage(cImage *pic)
{
    if (m_numPics >= MAX_PICS)
        return false;

    m_pic[m_numPics] = pic;
    ++m_numPics;

    return true;
}

bool
cAnimation::Load(char *filename)
{
	FILE *fp;

	if (!(fp = fopen(filename, "rt")))
		return false;

	char dummy[20], picFile[50];
	int version;

	fscanf(fp, "%s", dummy);// Version:
	fscanf(fp, "%d", &version);

	fscanf(fp, "%s", dummy);// Length:
	fscanf(fp, "%d", &m_length);

	fscanf(fp, "%s", dummy);// Delay:
	fscanf(fp, "%d", &m_delay);

	fscanf(fp, "%s", dummy);// Repeat:
	fscanf(fp, "%s", dummy);// Yes/No
	if (!strcasecmp(dummy, "Yes"))
		m_repeat = true;

	fscanf(fp, "%s", dummy);//Frames:
	for (int i = 0; i < m_length; i++)
		fscanf(fp, "%d", &m_frame[i]);

	fscanf(fp, "%s", dummy);//NumPics:
	fscanf(fp, "%d", &m_numPics);

	switch (version)
	{
	case 1:
		for (int i = 0; i < m_numPics; i++) {
			fscanf(fp, "%s", dummy);//Pic#:
			fscanf(fp, "%s", picFile);
			m_pic[i] = new cImage(picFile);
		}
		break;
	case 2:
		fscanf(fp, "%s", dummy);//Pic:
		fscanf(fp, "%s", picFile);

		BITMAP *pic = load_bitmap(picFile, NULL);
		BITMAP *subPic;
//blit(BITMAP *source, BITMAP *dest, int source_x, int source_y, int dest_x, int dest_y, int width, int height);
		for (int i = 0; i < m_numPics; i++) {
			subPic = create_bitmap(pic->w/m_numPics, pic->h);
			blit(pic, subPic, i*pic->w/m_numPics, 0, 0, 0, pic->w/m_numPics, pic->h);
			m_pic[i] = new cImage(subPic);
		}
		destroy_bitmap(pic);
		break;
	}

	fclose (fp);

	return true;
}

void
cAnimation::Unload()
{
	if (m_loaded) {
		m_loaded = false;

		for (int i = 0; i < m_numPics; i++) {
			delete m_pic[i];
			m_pic[i] = NULL;
		}
	}
}

void
cAnimation::setPosition(int x, int y)
{
    xOffset = -x;
    yOffset = -y;
}
