#include "cPlayer.h"

#include "cImage.h"
#include "cGraphics.h"
#include "cAnimation.h"
#include "cTilemap.h"

cPlayer::cPlayer()
{
    for (int i = 0; i < 25; i++) {
    	m_oldX[i] = m_x;
    	m_oldY[i] = m_y;
    }
    m_state = eNORMAL;
    m_anim = new cAnimation("playerAnim.ani");
    m_anim->setOffset(-(400-32),-(300-32));
	m_deathWait = 0;
	m_lives = 5;
}

cPlayer::cPlayer(char *filename)
{
    for (int i = 0; i < 25; i++) {
    	m_oldX[i] = m_x;
    	m_oldY[i] = m_y;
    }
    m_state = eNORMAL;
    m_anim = new cAnimation(filename);
    m_anim->setOffset(-(400-32),-(300-32));
    m_deathWait = 0;
    m_lives = 5;
}

cPlayer::~cPlayer()
{
//    for (int i = 0; i < 4; i++)
//        if (pic[i])
//            delete pic[i];
    delete m_anim;
}

void
cPlayer::Draw()
{
    if (m_state == eNORMAL)
		m_anim->Draw();

//    rect(g_graphics.double_buffer,400-HALF_WIDTH,300+20,400+HALF_WIDTH,300+32,makecol(255,0,0));
//    circle(g_graphics.double_buffer,400,300,10,0);
}

void
cPlayer::Update() {

	switch (m_state)
	{
	case eNORMAL:
		int xTiles = m_currentLevel->m_xTiles;
		int yTiles = m_currentLevel->m_yTiles;

		if (key[KEY_UP]) {
			if (!m_currentLevel->isImpassable(m_x, m_y-10))
				m_y -= 10;

			if (m_y < 300)
				m_y = 300;
		}
		if (key[KEY_DOWN]) {
			if (!m_currentLevel->isImpassable(m_x, m_y+10))
				m_y += 10;

			if (m_y > yTiles*64-300)
				m_y = yTiles*64-300;
		}
		if (key[KEY_RIGHT]) {
			if (!m_currentLevel->isImpassable(m_x+10, m_y))
				m_x += 10;

			if (m_x > xTiles*64-400-16)
				m_x = xTiles*64-400-16;
		}
		if (key[KEY_LEFT]) {
			if (!m_currentLevel->isImpassable(m_x-10, m_y))
				m_x -= 10;

			if (m_x < 400+16)
				m_x = 400+16;
		}

		// Avoid compiler complaints
		{
			bool fall = true;

			if (m_currentLevel->getTileIndex(m_x-HALF_WIDTH, m_y+20) != 0 ||
				m_currentLevel->getTileIndex(m_x+HALF_WIDTH, m_y+20) != 0 ||
				m_currentLevel->getTileIndex(m_x-HALF_WIDTH, m_y+32) != 0 ||
				m_currentLevel->getTileIndex(m_x+HALF_WIDTH, m_y+32) != 0)
			{
				fall = false;
			}

			if (fall) {
				m_anim->Stop();
				m_state = eDEAD;
				--m_lives;
			}
		}

		for (int i = m_x - 64; i <= m_x + 64; i += 64)
			for (int j = m_y - 64; j <= m_y + 64; j += 64)
				m_currentLevel->BreakTile(i, j);

		m_anim->Update();

		// Update oldxy list
		for (int i = 23; i >= 0; i--) {
			m_oldX[i+1] = m_oldX[i];
			m_oldY[i+1] = m_oldY[i];
		}
		m_oldX[0] = m_x;
		m_oldY[0] = m_y;
		break;
	case eDEAD:
		float xStep, yStep;

		xStep = (m_oldX[24] - m_x) / 20.0;
		yStep = (m_oldY[24] - m_y) / 20.0;

		m_x = m_oldX[m_deathWait];
		m_y = m_oldY[m_deathWait];

		++m_deathWait;

		if (m_deathWait >= 25) {
			m_deathWait = 0;
			m_x = m_oldX[24];
			m_y = m_oldY[24];
			m_anim->Play(0,0);
			m_state = eNORMAL;
		}
		break;
	}

}

void
cPlayer::setCurrentLevel(cTilemap *level)
{
	m_currentLevel = level;
	level->getStart(&m_x, &m_y);
}

cAnimation
*cPlayer::getCurrentAnimation()
{
	return m_anim;
}
