// TINS 2008
// Kwame Alexander

#include "cGraphics.h"
#include "cImage.h"
#include "cPlayer.h"
#include "cAnimation.h"
#include "cTilemap.h"

#define PI_TO_256(x) (int)( (x+M_PI) * 256.0f / (2.0f*M_PI) )
#define PI_TO_64(x) (int)( (x+M_PI) * 64.0f / (2.0f*M_PI) )
#define PI_TO_256x6(x) (int)( (x+M_PI) * 6.0*256.0f / (2.0f*M_PI) )
#define PI_TO_RANGE(x,y) (int)( (x+M_PI) * y / (2.0f*M_PI) )

long framecounter;

static int *CreateCrazyPalette();
static int TitleScreen(int initialOption);

int main ()
{
    allegro_init();
	install_keyboard();
	install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL);
	srand(time(NULL));
	framecounter = 0;

	set_color_depth(16);
	if (!g_graphics.Initialize(cGraphics::FULLSCREEN)) {
        FILE *fp = fopen("scrap/error.txt","wt");
        time_t timer;
        time(&timer);
		fprintf(fp,"Oh crap. Time: %s", ctime(&timer));
		fclose(fp);
	}

	int option = 0;
	do
	{
		option = TitleScreen(option);

		switch (option)
		{
		case 0:
			break;
		case 2:
			clear_bitmap(g_graphics.double_buffer);
			g_graphics.DrawText (110, 100, 255, 255, 255, "You are Zerog, a monster who wishes to join the League of Forogth,");
			g_graphics.DrawText (110, 110, 255, 255, 255, "a well-known monstrous organization that wishes to take over the");
			g_graphics.DrawText (110, 120, 255, 255, 255, "underworld. To join them, you must pass their test!");
			g_graphics.DrawText (110, 140, 255, 255, 255, "Very few survive.");
			g_graphics.DrawText (110, 160, 255, 255, 255, "The test will be administered by Squarg, a senior member of the");
			g_graphics.DrawText (110, 170, 255, 255, 255, "League.");
			g_graphics.DrawText (110, 190, 255, 255, 255, "He is not very nice.");
			g_graphics.DrawText (110, 230, 128, 128, 255, "Press any key to return to the title...");
			g_graphics.DrawBuffer();

			while (keypressed())
				clear_keybuf();
			while (!keypressed());

			break;
		case 1:
			clear_bitmap(g_graphics.double_buffer);
			g_graphics.DrawText (110, 100, 255, 255, 255, "Use the arrow keys to move. Make your way to the teleporter without");
			g_graphics.DrawText (110, 110, 255, 255, 255, "falling into the water, lava, or whatever trap that lies beneath the");
			g_graphics.DrawText (110, 120, 255, 255, 255, "surface.");
			g_graphics.DrawText (110, 130, 255, 255, 255, "Good luck.");
			g_graphics.DrawText (110, 170, 128, 128, 255, "Press any key to return to the title...");
			g_graphics.DrawBuffer();

			while (keypressed())
				clear_keybuf();
			while (!keypressed());

			break;
		case 3: // Quit
			g_graphics.UnloadModule();
			return (0);
		}
	} while (option != 0);

    cPlayer *player = new cPlayer();

	char levelFile[3][30] = {"volcano.lvl", "snow.lvl", "space.lvl"};

	cAnimation *teleporter, *headAnim, *headDead;
    teleporter = new cAnimation("teleporter.ani");
    int currentLevel = 0;
    headAnim = new cAnimation("headAnim.ani");
	int head[10];

	headDead = new cAnimation("headDead.ani");

	SAMPLE *scream[3];
	scream[0] = load_wav("media/scream0.wav");
	scream[1] = load_wav("media/scream1.wav");
	scream[2] = load_wav("media/scream2.wav");

	while (player->m_lives > 0 && currentLevel < 3)
    {
		cTilemap *tilemap = new cTilemap(levelFile[currentLevel]);

		player->setCurrentLevel(tilemap);
		int goalTileX, goalTileY;
		tilemap->getGoalTile(&goalTileX, &goalTileY);
		teleporter->Play(goalTileX*64,goalTileY*64-32);
		teleporter->setOffset(player->m_x-400,player->m_y-300);

		player->getCurrentAnimation()->Play(0,0);

		int lives = player->m_lives;

		cAnimation *npc = new cAnimation("NPC.ani");
		npc->Play(0,0,2);
		int npcY = 0, npcState = 0;
		bool spacePressed = false;

		while (npcState != 3) {

			if (key[KEY_ESC]) {
				npcState = 3;
				clear_keybuf();
			}

			int timekeeper = clock() + CLOCKS_PER_SEC/25;

			tilemap->Draw(player->m_x-400, player->m_y-300);

			npc->setPosition(300, npcY);
			npc->Update();
			npc->Draw();

			teleporter->Update();
			teleporter->Draw();

			player->getCurrentAnimation()->Update();
			player->getCurrentAnimation()->Draw();

			switch (npcState)
			{
			case 0:
				if (key[KEY_SPACE] && !spacePressed) {
					npcState = 1;
					spacePressed = true;
				}
				if (!key[KEY_SPACE]) {
					spacePressed = false;
				}

				npcY += 10;

				if (npcY >= 300-32)
					npcState = 1;

				break;
			case 1:
				if (key[KEY_SPACE] && !spacePressed) {
					npcState = 2;
					spacePressed = true;
				}
				if (!key[KEY_SPACE]) {
					spacePressed = false;
				}

				//show text
				int numLines = tilemap->getNumLines();
				int maxLength = tilemap->getMaxLineLength();
				rectfill(g_graphics.double_buffer,100,20,120+8*maxLength,40+10*numLines,makecol(64,64,192));
				rect(g_graphics.double_buffer,100,20,120+8*maxLength,40+10*numLines,makecol(255,255,255));
				for (int i = 0; i < numLines; i++) {
					g_graphics.DrawText (110, 30+10*i, 255, 255, 255, tilemap->getLine(i));
				}

				break;
			case 2:
				if (key[KEY_SPACE] && !spacePressed) {
					npcState = 3;
					spacePressed = true;
				}
				if (!key[KEY_SPACE]) {
					spacePressed = false;
				}
				npcY -= 10;

				if (npcY < -64) {
					npcState = 3;
					npc->Stop();
				}

				break;
			}

			g_graphics.DrawBuffer();
			while (timekeeper > clock());
			++framecounter;
		}
		delete npc;

		for (int i = 0; i < player->m_lives; i++) {
			head[i] = headAnim->Play(20 + i*25, 20, i);
		}

		bool madeGoal = false, gaveUp = false;

		while (player->m_lives > 0 && !madeGoal) {
			int timekeeper = clock() + CLOCKS_PER_SEC/25;

			tilemap->Draw(player->m_x-400, player->m_y-300);

			teleporter->setOffset(player->m_x-400,player->m_y-300);
			teleporter->Update();
			teleporter->Draw();

			if (lives > player->m_lives) {
				--lives;
				headAnim->Stop(head[lives]);
				headDead->Play(20-9 + lives*25, 20-9);
				play_sample(scream[rand()%3], 128, 128, 800, 0);
			}

			if (tilemap->isGoal(player->m_x, player->m_y)) {
				player->getCurrentAnimation()->Stop();
				madeGoal = true;
			}

			if (key[KEY_ESC]) {
				gaveUp = true;
				break;
			}

			headAnim->Update();
			headAnim->Draw();

			headDead->Update();
			headDead->Draw();

			player->Update();
			player->Draw();

			g_graphics.DrawBuffer();

			while (timekeeper > clock());

			++framecounter;
		}

		teleporter->Stop();
		for (int i = 0; i < player->m_lives; i++)
			headAnim->Stop(head[i]);

		delete tilemap;

		if (gaveUp)
			break;

		++currentLevel;

		if (player->m_lives > 0)
			++player->m_lives;
    }

    if (currentLevel == 3 && player->m_lives > 0) {
		clear_bitmap(g_graphics.double_buffer);
		g_graphics.DrawText (110, 110, 255, 255, 255, "Congratulations! You are now a member of the League of Forogth");
		g_graphics.DrawText (110, 120, 255, 255, 255, "and will spend the rest of your happy life, making things difficult");
		g_graphics.DrawText (110, 130, 255, 255, 255, "for the other denizens of the underworld.");
		g_graphics.DrawBuffer();

		while(keypressed())
			clear_keybuf();
		while (!keypressed());
    }
    if (player->m_lives <= 0) {
    	clear_bitmap(g_graphics.double_buffer);
		g_graphics.DrawText (110, 110, 255, 255, 255, "You did not pass. You are not part of the League.");
		g_graphics.DrawText (110, 120, 255, 255, 255, "In fact, you are not alive any longer.");
		g_graphics.DrawText (110, 150, 255, 0, 0, "Game Over");
		g_graphics.DrawBuffer();

		while(keypressed())
			clear_keybuf();
		while (!keypressed());
    }

    delete teleporter;
	delete headDead;
	delete headAnim;
    for (int i = 0; i < 3; i++) {
        destroy_sample(scream[i]);
    }
    delete player;
    g_graphics.UnloadModule();

    return (0);
}
END_OF_MAIN()

static int
*CreateCrazyPalette()
{
	int *palette;

	palette = (int *)malloc(6*256*sizeof(int));

	// Red to yellow
	int start = 0;
	for (int i = start; i < 256; i++) {
		palette[i] = makecol(255, i, 0);
	}
	//Yellow to green
	start = 256;
	for (int i = start; i < 256*2; i++) {
		palette[i] = makecol(255-(i-start), 255, 0);
	}
	//Green to cyan
	start = 256*2;
	for (int i = start; i < 256*3; i++) {
		palette[i] = makecol(0, 255, i-start);
	}
	//Cyan to blue
	start = 256*3;
	for (int i = start; i < 256*4; i++) {
		palette[i] = makecol(0, 255-(i-start), 255);
	}
	//Blue to magenta
	start = 256*4;
	for (int i = start; i < 256*5; i++) {
		palette[i] = makecol(i-start, 0, 255);
	}
	//Magenta to red
	start = 256*5;
	for (int i = start; i < 256*6; i++) {
		palette[i] = makecol(255, 0, 255-(i-start));
	}

	return palette;
}

static int
TitleScreen(int initialOption)
{
	int *palette;
	palette = CreateCrazyPalette();

	clear_bitmap(g_graphics.double_buffer);

    for (int x = -150; x < 150; x++) {
        for (int y = -150; y < 150; y++) {
        	if (x*x+y*y <= 150*150)
				putpixel(g_graphics.double_buffer, x+400, 300-y, palette[PI_TO_RANGE(atan2(x,y),255*6)]);
        }
    }
    free(palette);
    cImage *zero = new cImage("media/zero.bmp");
    zero->DrawCentred(400,300);
    delete zero;

	cImage *titleOptions = new cImage("media/TitleOptions.bmp");
	titleOptions->Draw(0,0);
	delete titleOptions;

	cImage *pointer = new cImage("media/faerie_fire.pcx");

	int pointerPos[4] = {90,278,470,574}, option = initialOption;
	bool rightPressed = false, leftPressed = false;

	while (keypressed())
		clear_keybuf();

	while (!key[KEY_ENTER] && !key[KEY_SPACE]) {
		if (key[KEY_LEFT] && !leftPressed) {
			circlefill(g_graphics.double_buffer, pointerPos[option], 35, 13, 0);

			--option;
			if (option < 0)
				option = 3;

			leftPressed = true;
		}
		if (!key[KEY_LEFT] && leftPressed) {
			leftPressed = false;
		}

		if (key[KEY_RIGHT] && !rightPressed) {
			circlefill(g_graphics.double_buffer, pointerPos[option], 35, 13, 0);

			++option;
			if (option > 3)
				option = 0;

			rightPressed = true;
		}
		if (!key[KEY_RIGHT] && rightPressed) {
			rightPressed = false;
		}

		if (key[KEY_ESC])
			return(3);

		pointer->DrawCentred(pointerPos[option], 35);
		g_graphics.DrawBuffer();
	}

	return option;
}
